using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace PostProcessing
{
    public class GridVertexPositionColorTexture
    {
        /// <summary>
        /// vertices of the grid
        /// </summary>
        public VertexPositionColorTexture[] Vertices { get; set; }

        /// <summary>
        /// Indices of the grid
        /// </summary>
        public short[] Indices { get; set; }

        /// <summary>
        /// Width of the grid
        /// </summary>
        public int Width { get; protected set; }

        /// <summary>
        /// Height of the grid
        /// </summary>
        public int Height { get; protected set; }

        /// <summary>
        /// Size (in pixels) of a cell
        /// </summary>
        public Vector2 CellSize { get; protected set; }

        /// <summary>
        /// Builds the grid
        /// </summary>
        /// <param name="colums">number of columns</param>
        /// <param name="rows">number of rows</param>
        /// <param name="cellSize">size of each cell</param>
        public void BuildGeometry(int colums, int rows, Vector2 cellSize)
        {
            Width = colums;
            Height = rows;
            CellSize = cellSize;

            Vertices = new VertexPositionColorTexture[(Width + 1) * (Height + 1)];
            Indices = new short[Width * Height * 6];

            for (int i = 0; i < Width + 1; i++)
            {
                for (int j = 0; j < Height + 1; j++)
                {
                    int index = j * (Width + 1) + i;
                    VertexPositionColorTexture vertex = new VertexPositionColorTexture()
                    {
                        Position = new Vector3(new Vector2(i, j) * CellSize, 0),
                        Color = Color.White,
                        TextureCoordinate = GetDefaultUV(index)
                    };
                    Vertices[index] = vertex;
                }
            }

            int indexPos = 0;
            for (int i = 0; i < Width; i++)
            {
                for (int j = 0; j < Height; j++)
                {
                    int v0 = j * (Width + 1) + i;
                    int v1 = j * (Width + 1) + i + 1;
                    int v2 = (j + 1) * (Width + 1) + i;
                    int v3 = (j + 1) * (Width + 1) + i + 1;

                    Indices[indexPos] = (short)v0;
                    Indices[indexPos + 1] = (short)v1;
                    Indices[indexPos + 2] = (short)v2;
                    Indices[indexPos + 3] = (short)v2;
                    Indices[indexPos + 4] = (short)v1;
                    Indices[indexPos + 5] = (short)v3;
                    indexPos += 6;
                }
            }
        }

        /// <summary>
        /// Draw the grid, assuming an effect is already set-up
        /// </summary>
        /// <param name="graphicsDevice"></param>
        public void Draw(GraphicsDevice graphicsDevice)
        {
            graphicsDevice.DrawUserIndexedPrimitives<VertexPositionColorTexture>(PrimitiveType.TriangleList, Vertices, 0, Vertices.Length, Indices, 0, Indices.Length / 3);
        }

        /// <summary>
        /// Reset all UVs
        /// </summary>
        public void ResetUVs()
        {
            for (int i = 0; i < Vertices.Length; i++)
            {
                VertexPositionColorTexture v = Vertices[i];
                v.TextureCoordinate = GetDefaultUV(i);
                Vertices[i] = v;
            }
        }

        /// <summary>
        /// Get Texture Coordinate of the element at position <b>index</b>
        /// </summary>
        public Vector2 GetUV0(int index)
        {
            return Vertices[index].TextureCoordinate;
        }

        /// <summary>
        /// Set Texture Coordinate of the element at position <b>index</b>
        /// </summary>
        public void SetUV0(int index, Vector2 value)
        {
            Vertices[index].TextureCoordinate = value;
        }

        /// <summary>
        /// Get the default Texture Coordinate of the element at position <b>index</b>
        /// </summary>
        public Vector2 GetDefaultUV(int index)
        {
            int i = index % (Width + 1);
            int j = index / (Width + 1);
            return new Vector2((float)i / Width, (float)j / Height);
        }
    }
}
